import { Reflector } from '@nestjs/core'
import {
  Injectable,
  CanActivate,
  ExecutionContext,
  applyDecorators,
  UseGuards,
  SetMetadata,
} from '@nestjs/common'
import { EmailScene, EmailVerifyDto } from './dto/email.dto'
import { EmailService } from './email.service'
import { type Request } from 'express'

const EMAIL_SCENE_KEY = 'EMAIL_SCENE'

@Injectable()
class EmailVeriyGuard implements CanActivate {
  constructor(
    private emailService: EmailService,
    private reflector: Reflector,
  ) {}

  async canActivate(ctx: ExecutionContext) {
    const scene = this.reflector.get<EmailScene>(
      EMAIL_SCENE_KEY,
      ctx.getHandler(),
    )

    const body: EmailVerifyDto = ctx.switchToHttp().getRequest<Request>().body
    await this.emailService.verifyEmailCode({ scene, ...body })
    return true
  }
}

export function VerifyEmail(scene: EmailScene) {
  return applyDecorators(
    SetMetadata(EMAIL_SCENE_KEY, scene),
    UseGuards(EmailVeriyGuard),
  )
}
